// Procedure File Manager Panel
// Jeffrey J Weimer
// SVN History: $Revision: 871 $ on $Date: 2008-07-24 18:25:37 -0500 (Thu, 24 Jul 2008) $

#pragma rtGlobals=1
#pragma version = 2.0

#pragma ModuleName=PFManager

#include "PFManager.lcl"

// Procedure String Constants
// define the default file folder and its name

Static StrConstant kProcFileFolder="PFManager"
Static StrConstant kProcFileName="PFManager.ipf"
Static StrConstant kPackageName="PFManager"

// TEST MODE (comment out for actual use)
#define TESTME

Menu "Misc", hideable
	Submenu "Panels"
		"Procedure File Manager",/Q,PFManager#ShowPanel()
	end
End

 Initialize Stuff

Static Function InitializeStructure(sPFM)
	STRUCT PFManagerStructure &sPFM
	sPFM.ph = 130
	sPFM.pw = 240
end

Static STRUCTURE PFManagerStructure
	variable ph
	variable pw
ENDSTRUCTURE

// Read Preferences

Static Function RestorePackagePrefs()

	Variable result = -1, ic
	string theOne
	String saveDF = GetDataFolder(1)

	NewDataFolder/O/S root:Packages	// Ensure root:Packages exists
	NewDataFolder/O/S $kpackageName	// Ensure package data folder exists

	do
		theOne = GetIndexedObjName("",4,ic)
		if (strlen(theOne)==0)
			break
		endif
		KillDataFolder $theOne
		ic += 1
	while(1)
	
	// Find the disk directory in the Packages directory for this package
	String fullPath = SpecialDirPath("Packages", 0, 0, 0)
	fullPath += kpackageName
	GetFileFolderInfo/Q/Z fullPath
	if (V_Flag == 0)				// Disk directory exists?
		fullPath += ":Preferences.pxp"
		GetFileFolderInfo/Q/Z fullPath
		if (V_Flag == 0)			// Preference file exist?
			LoadData/O/R/Q fullPath		// Load the preference file.
			result = 0
		endif
	endif
	
	SetDataFolder saveDF
	return result
end

// Save Preferences

Static Function SavePackagePrefs()

	// Get path to Packages preferences directory on disk.

	string fullPath, cdf

	fullPath = SpecialDirPath("Packages", 0, 0, 0) + kPackageName
	NewPath/O/C/Q tempPackagePrefsPath, fullPath

	fullPath += ":Preferences.pxp"

	cdf = GetDataFolder(1)

	SetDataFolder root:Packages:$kPackageName
	SaveData/O/Q/R fullPath

	SetDataFolder cdf
	KillPath/Z tempPackagePrefsPath
	return 0
End

// Show the Panel

Static Function ShowPanel()

	if (strlen(WinList("PFM_Panel","","WIN:64"))!=0)
		DoWindow/F PFM_Panel
		return 0
	endif
	
	string cdf = GetDataFolder(1)
	NewDataFolder/O/S root:Packages
	NewDataFolder/O/S $kPackageName
	
	string/G ModeList, NewModeStr
	variable/G PanelSize
	
	SVAR nmstr = NewModeStr
	SVAR mlist = ModeList
	NVAR psize = PanelSize
	
	nmstr = ""
	mlist = ""
	
	variable ic
	do
		nmstr = GetIndexedObjName("", 4, ic )
		if (strlen(nmstr)==0)
			break
		endif
		mlist += nmstr + ";"
		ic +=1
	while(1)
	
	STRUCT PFManagerStructure sPFM
	InitializeStructure(sPFM)
	
	if (sPFM.ph<130)
		sPFM.ph=130
	endif
	psize = sPFM.ph
	
	make/T/O/N=(0) Cpackages=""
	make/O/N=(0) CpackageSEL=32

	SetDataFolder $cdf
	
	PutUpPanel(sPFM)

	string exeStr
	sprintf exeStr, "SetIgorOption IndependentModuleDev = %g", 0
	Execute/Q exeStr
	
//	ToggleMeInvisible(1)
//	ToggleLocalInvisible(1)	

	return 0
end

Static Function PutUpPanel(spFM)
	STRUCT PFManagerStructure &sPFM
	
	NVAR psize = $("root:Packages:" + kPackageName + ":" + "PanelSize")
	SVAR nmstr = $("root:Packages:" + kPackageName + ":" + "NewModeStr")
	
	wave CPackageSEL = $("root:Packages:" + kPackageName + ":CPackageSEL")	
	wave/T CPackages = $("root:Packages:" + kPackageName + ":CPackages")		
	
#if (exists("ScreenSizer#Initialize")==6)	
	ScreenSizer#Initialize(0)
	ScreenSizer#MakePanel(-5,5,sPFM.pw,sPFM.ph,"PF Manager + SS",other="/K=1/N=PFM_Panel")
#else
	NewPanel/W=(10,10,sPFM.pw,sPFM.ph)/K=1/N=PFM_Panel as "PF Manager"
#endif

#ifndef TESTME
	ModifyPanel fixedSize=1, noEdit=1
#endif
	
	variable lh, lw, pz = 20
	
	lh = spFM.ph - 85
	lw = sPFM.pw - 10

	// Tabs
	
	TabControl tab0,pos={0,0},size={sPFM.pw,20},tabLabel(0)="Use",tabLabel(1)="Define"
	TabControl tab0,value=0, proc=PFManager#ChangeTab

	// Mode
	
	Button tb1_delmode,pos={3,pz+2}, size={15,15}, title="-"
	Button tb1_delmode,proc=PFManager#DeleteMode
	PopupMenu mode title="Mode",pos={20,pz},value=#"PFManager#ModeList()", mode=1, proc=PFManager#ToggleMode

	// restore button
	Button tb1_restore,pos={195,pz + 2}, size={35,20}, title="Get"
	Button tb1_restore, proc=PFManager#GetModes
	
	// package list
	
	ListBox list0 mode=0,font="default", listwave=Cpackages, selwave=CpackageSEL
	ListBox list0 pos={5,pz+25},size={lw,lh},frame=0,font="default"

	// insert with/without compile
	CheckBox tb0_check0,pos={5,pz + 25 + lh + 5},size={165,15},title="Compile After Insert"
	CheckBox tb0_check0,value= 1
	// independent module development
	CheckBox tb0_check1,pos={5,pz + 25 + lh + 20},size={165,15},title="Indepent Module Development"
	CheckBox tb0_check1,value= 0,proc=PFManager#SetIMDevelopment
	// save button
	Button saveX,pos={195,pz + 25 + lh + 2},picture=PFManager#SaveXButton
	Button saveX, proc=PFManager#SaveThisExperiment, title=""

	Button tb1_addfile,pos={5,pz + 25 + lh + 1}, size={65,20}, title="Add"
	Button tb1_addfile,proc=PFManager#AddNewFile
	
	Button tb1_delfile,pos={70,pz + 25 + lh + 1}, size={65,20}, title="Remove"
	Button tb1_delfile,proc=PFManager#RemoveFile

	SetVariable tb1_newmode, value=nmstr, pos= {20,sPFM.ph-20}, size={lw-80,20}, title=" "	
	Button tb1_addmode,pos={3,sPFM.ph-18}, size={15,15}, title="+"
	Button tb1_addmode,proc=PFManager#AddNewMode

	SetVariable tb1_size,pos={145,pz + 25 + lh + 1},size={50,18},proc=PFManager#SetPanelSize,title=" "
	SetVariable tb1_size,limits={130,400,10},value=psize	

	SetIgorHook IgorQuitHook=PFManager#CleanUpBeforeQuit
	ToggleControlsVisibleState(0)	
	return 0
end

Static Function SetPanelSize(sva) : SetVariableControl
	STRUCT WMSetVariableAction &sva

	switch( sva.eventCode )
		case 1: // mouse up
		case 2: // Enter key
			ResetPanelSize(sva.dval)
			break
	endswitch

	return 0
End

Static Function ResetPanelSize(size)
	variable size
	
	STRUCT PFManagerStructure sPFM
	InitializeStructure(sPFM)
	sPFM.ph = size
	
	KillWindow PFM_Panel
	PutUpPanel(sPFM)
	
	return 0
end

Static Function AddNewMode(ba) : ButtonControl
	STRUCT WMButtonAction &ba
	
	SVAR ModeList = $("root:Packages:" + kPackageName + ":ModeList")	
	SVAR NewMode = $("root:Packages:" + kPackageName + ":NewModeStr")
	
	wave CPackageSEL = $("root:Packages:" + kPackageName + ":CPackageSEL")	
	wave/T CPackages = $("root:Packages:" + kPackageName + ":CPackages")

	variable vts
	string mdf
	
	switch( ba.eventCode )
		case 2:
			if (strlen(NewMode)==0)
				return 0
			endif
			vts = WhichListItem(NewMode,ModeList)
			mdf = "root:Packages:" + kPackageName + ":" + PossiblyQuoteName(NewMode)
			if (vts<0)
				if (strlen(ModeList)!=0)
					SaveCurrentMode()
				endif
				ModeList += NewMode + ";"
				vts = ItemsInList(ModeList)
				Redimension/N=0 CPackages, CPackageSEL
				Button tb1_addfile, disable=0
				Button tb1_delfile, disable=0
				NewDataFolder/O $mdf
			else
				vts += 1
			endif
			PopupMenu mode value=#"PFManager#ModeList()", popValue=NewMode, mode=vts		
			NewMode = ""
			break
	endswitch
	
	return 0
end

Static Function SaveCurrentMode()
	
	wave CPackageSEL = $("root:Packages:" + kPackageName + ":CPackageSEL")	
	wave/T CPackages = $("root:Packages:" + kPackageName + ":CPackages")

	string mdf = "root:Packages:" + kPackageName + ":"
	ControlInfo mode
	mdf += PossiblyQuoteName(S_value)
	
	string cpStore = mdf + ":CPackages"
	string cpSELStore = mdf + ":CPackageSEL"
	
	duplicate/O/T CPackages $cPStore
	duplicate/O CPackageSEL $cPSELStore
	
	return 0
end

Static Function DeleteMode(ba) : ButtonControl
	STRUCT WMButtonAction &ba
	
	SVAR ModeList = $("root:Packages:" + kPackageName + ":ModeList")	
	
	wave CPackageSEL = $("root:Packages:" + kPackageName + ":CPackageSEL")	
	wave/T CPackages = $("root:Packages:" + kPackageName + ":CPackages")

	variable vtr
	string mdf
	switch( ba.eventCode )
		case 2:
			ControlInfo mode
			vtr = V_value-1
			ModeList = RemoveListItem(vtr,ModeList)
			PopupMenu mode value=#"PFManager#ModeList()", popValue=StringFromList(0,ModeList), mode=1
			Redimension/N=0 CPackageSEL, CPackages
			mdf = "root:Packages:" + kPackageName + ":" + PossiblyQuoteName(S_value)
			KillDataFolder $mdf
			break
	endswitch
	
	return 0
end

Static Function GetModes(ba) : ButtonControl
	STRUCT WMButtonAction &ba
	
	switch( ba.eventCode )
		case 2:
			DoAlert 1, "Getting the modes list overwrites the current values!!!\rDo you really want to do this?"
			if (V_flag==1)
				RestorePackagePrefs()
				SVAR ModeList = $("root:Packages:" + kPackageName + ":ModeList")	
				NVAR psize = $("root:Packages:" + kPackageName + ":PanelSize")	
				PopupMenu mode value=#"PFManager#ModeList()", popValue=StringFromList(0,ModeList), mode=1
				ResetPanelSize(psize)
			endif
			break
	endswitch
	return 0
end

Static Function AddNewFile(ba) : ButtonControl
	STRUCT WMButtonAction &ba
	
	SVAR ModeList = $("root:Packages:" + kPackageName + ":ModeList")	
	
	wave CPackageSEL = $("root:Packages:" + kPackageName + ":CPackageSEL")	
	wave/T CPackages = $("root:Packages:" + kPackageName + ":CPackages")

	switch( ba.eventCode )
		case 2:
			if (strlen(ModeList)==0)
				return 0
			endif
			InsertPoints 0,1, CPackages, CPackageSEL
			CPackages[0] = ""
			CPackageSEL[0] = 2
			Listbox list0 selrow=1, activate
			break
	endswitch
	return 0
end

Static Function RemoveFile(ba) : ButtonControl
	STRUCT WMButtonAction &ba
	
	SVAR ModeList = $("root:Packages:" + kPackageName + ":ModeList")	
	
	wave CPackageSEL = $("root:Packages:" + kPackageName + ":CPackageSEL")	
	wave/T CPackages = $("root:Packages:" + kPackageName + ":CPackages")

	switch( ba.eventCode )
		case 2:
			if (strlen(ModeList)==0)
				return 0
			endif
			ControlInfo list0
			DeletePoints V_value, 1, CPackageSEL, CPackages
			break
	endswitch
	return 0
end


Static Function ChangeTab(tca) : TabControl
	STRUCT WMTabControlAction &tca
	
	SVAR ModeList = $("root:Packages:" + kPackageName + ":ModeList")	

	switch( tca.eventCode )
		case 2: // mouse up
//			if (strlen(ModeList)!=0)
//				if (tca.tab==1)
//					SaveCurrentMode()
//				endif
//			endif
			ToggleControlsVisibleState(tca.tab)
			break
	endswitch

	return 0
End

Static Function ToggleControlsVisibleState(tab)
	variable tab
	
	SVAR ModeList = $("root:Packages:" + kPackageName + ":ModeList")	
	
	wave CPackageSEL = $("root:Packages:" + kPackageName + ":CPackageSEL")
	
	switch(tab)
		case 0:
			ModifyControlList ControlNameList("",";","tb1*") disable=1
			ModifyControlList ControlNameList("",";","tb0*") disable=0
			Button saveX, proc=PFManager#SaveThisExperiment
			ListBox list0 mode=0, proc=PFManager#TogglePackage
			ControlInfo mode
			string mdf = "root:Packages:" + kPackageName + ":" + PossiblyQuoteName(S_value)
			wave cPSEL = $(mdf + ":CPackageSEL")
			duplicate/O cPSEL CPackageSEL
			break
		case 1:
			SaveCurrentMode()
			ModifyControlList ControlNameList("",";","tb0*") disable=1
			ModifyControlList ControlNameList("",";","tb1*") disable=0
			ListBox list0 noProc,mode=1
			CpackageSEL = 2
			Button saveX, proc=PFManager#SaveTheModes
			if (strlen(ModeList)==0)
				Button tb1_addfile, disable=2
				Button tb1_delfile, disable=2
			else
				Button tb1_addfile, disable=0
				Button tb1_delfile, disable=0
			endif
			break
	endswitch
	
	return 0	
end

Static Function/S ModeList()
	
	SVAR ModeList = $("root:Packages:" + kPackageName + ":ModeList")
	
	if (strlen(ModeList)==0)
		return "None"
	else
		return ModeList
	endif
end

Static Function CleanUpBeforeQuit(iAStr)
	string iAStr
	
	string exeStr
	sprintf exeStr, "SetIgorOption IndependentModuleDev = %g", 0
	Execute exeStr
	return 0
end

// Control Support Functions Below Here

Static Function ToggleMode(pa) : PopupMenuControl
	STRUCT WMPopupAction &pa
	
	wave CPackageSEL = $("root:Packages:" + kPackageName + ":CPackageSEL")	
	wave/T CPackages = $("root:Packages:" + kPackageName + ":CPackages")

	string mdf
	
	switch( pa.eventCode )
		case 2: // mouse up
			mdf = "root:Packages:" + kPackageName + ":" + PossiblyQuoteName(pa.popStr)
			ControlInfo tab0
			if (V_value == 0)
				wave cPSEL = $(mdf + ":CPackageSEL")
				duplicate/O cPSEL CPackageSEL
				wave/T cP = $(mdf + ":CPackages")
				duplicate/O/T cP CPackages
//				CPackageSEL = 32
				ListBox list0 mode=0, proc=PFManager#TogglePackage
			else
				wave cPSEL = $(mdf + ":CPackageSEL")
				duplicate/O cPSEL CPackageSEL
				wave/T cP = $(mdf + ":CPackages")
				duplicate/O/T cP CPackages
				CPackageSEL = 2
				ListBox list0 noProc,mode=1
			endif
			break
	endswitch

	return 0
End

Static Function SaveTheModes(ba) : ButtonControl
	STRUCT WMButtonAction &ba

	switch( ba.eventCode )
		case 2: // mouse up
			DoAlert 1, "Saving the modes list overwrites any previous values!!!\rDo you really want to do this?"
			if (V_flag==1)
				SavePackagePrefs()
			endif
			break
	endswitch

	return 0
End

Static Function SaveThisExperiment(ba) : ButtonControl
	STRUCT WMButtonAction &ba

	switch( ba.eventCode )
		case 2: // mouse up
			SaveExperiment
			break
	endswitch

	return 0
End

Static Function SetIMDevelopment(cba) : CheckBoxControl
	STRUCT WMCheckboxAction &cba

	string exeStr = ""
	switch( cba.eventCode )
		case 2:
			sprintf exeStr, "SetIgorOption IndependentModuleDev = %g", cba.checked
			Execute exeStr			
			break
	endswitch

	return 0
End

Function TogglePackage(lba) : ListBoxControl
	STRUCT WMListboxAction &lba

	Variable row = lba.row
	WAVE/T/Z listWave = lba.listWave
	WAVE/Z selWave = lba.selWave

	switch( lba.eventCode )
		case 2:
			variable nitems = numpnts(listWave) - 1
			if (row>nitems)
				break
			endif
			SaveCurrentMode()
			LoadUnloadPackage(listWave[row], selWave[row])
			break
	endswitch

	return 0
End

// Toggle Me Invisible
// 0: show procedure in folder and procedure menus
// 1: hide procedure in folder and procedure menus

Static Function ToggleMeInvisible(how)
	variable how

//	string IgorProcPathStr
//	PathInfo Igor
//	sprintf IgorProcPathStr, "%sUser Procedures:%s:", S_path, kProcFileFolder
//	NewPath/O/Q/Z TMP_ProcPath IgorProcPathStr
//	GetFileFolderInfo/Q/P=TMP_ProcPath kProcFileName
//	if (V_flag==0)
//		if (V_isInvisible!=how)
//			SetFileFolderInfo/P=TMP_ProcPath/INV=(how)/RO=(how) kProcFileName
//		endif
//	endif
//	KillPath/Z TMP_ProcPath
	return 0
end

//Function ShowLocalization(cba) : CheckBoxControl
//	STRUCT WMCheckboxAction &cba
//
//	switch( cba.eventCode )
//		case 2: // mouse up
//			if (cba.checked)
//				DoAlert 0, "Localization will reappear after a restart of Igor Pro"
//			else
//				DoAlert 0, "Localization will disappear after a restart of Igor Pro"
//			endif
//			ToggleLocalInvisible(!cba.checked)
//			break
//	endswitch
//
//	return 0
//End

Function ToggleLocalInvisible(how)
	variable how

//	string IgorProcPathStr, filePath
//	PathInfo Igor
//	IgorProcPathStr = S_path + "User Procedures:CodeDeveloperPanel:"
//	NewPath/O/Q/Z CDP_ProcPath IgorProcPathStr
//	filePath="CodeDeveloperPanel.lcl.ipf"
//	GetFileFolderInfo/Q/P=CDP_ProcPath filePath
//	if (V_flag==0)
//		SetFileFolderInfo/P=CDP_ProcPath/INV=(how)/RO=(how) filePath
//	endif
//	KillPath/Z CDP_ProcPath
	return 0
end

Static Function LoadUnloadPackage(itsName,itsValue)
	string itsName
	variable itsValue
	
	string exeCmd=""
	if (itsValue == 32)
//		print "unloading"
		exeCmd = "DELETEINCLUDE \"" + itsName +"\""
	else
//		print "loading"
		exeCmd = "INSERTINCLUDE \"" + itsName +"\""
	endif
	Execute/P/Q/Z exeCmd
	ControlInfo tb0_check0
	if (V_Value || (itsValue ==32))
		Execute/P/Q "COMPILEPROCEDURES "
	endif
	return 0
	
end

// PNG: width= 125, height= 39
Static Picture SaveXButton
	ASCII85Begin
	M,6r;%14!\!!!!.8Ou6I!!!"A!!!!E#R18/!4UO%mJm4e"EQn$<!Wa8#^cngL]@DT$#iF<ErZ1Lf)P
	d[f)\2dRs4o5!#]lI=EIJ[Aor6*Eb,5pGBYZRDJO<'@;od1DfTK[mQ;Bf!"CeW6pXdsSB[PLD]D8ja
	$3SS^SaT_L_NPr+De1=!&D2"TY88J/-A1]Y6bVI04#cV`<*Rr^gccST-.N[6&rJZJYhCEYRki:J-E$
	./R+)Ib;)aa'J3N/H@GN\GLt$3VgE#H%`K;W'YcnCHE-:4f=0V;pKgk#CY,]q-P5a[pE$e]h>5/7#_
	(bmc=t==>X/<<V:9MY778EIF31^=<D#D/`dF$sM<Xd3%pH)9gKL'fB7CE/V+[.oTGYJ.KI^-,!"_TH
	:HZt8i-F00>3eFhXT1cM=@iZA9iKGC-e@!gXYn+ck.,Qr*c%mCj#G*7SV(qTl;[gZ7W5.hCOOnRZg,
	-iRasM>PL*e_<CFiE77<q1(^q+_WO5J$F'O-]"Vn#=<pO>jj$5#h>c!Z<,,)EQ,5&oJW^<ZsWFP30&
	b,m'6o:RJh(Dms(YbJuEM?#.PdOP'=f*Sm6311=b)1_.#$ratXCEbY?=b-G4iK9%WFT.$Q=WZVeG,K
	hl#`Ne'q[]c1oVZcT'$)UCCq-]Ydl()/6O=\\Y&rC;7NbM&mLGQ^#$&4+j5NiZ[cGB_K?<JUX<7>OI
	G>m=dB(^87ndK+\bakSHOn6HHZTTOc\b:;G&AM;:\o^X]6Q!89<>IA[FNZrI0dS:l?<.2.qm#!or`,
	8WqHh><8=6DDsr$iqo0(FpUMPf:<V&CMhUo4O:)-jE[n5f2XS4q+J_+8c?-u][-W7pmQsK>?no?f5i
	fio4H,es*,?1\)719%mKbrjuQ*`c)i9d/aZ<QpHK("[kiD5@@%\"DfAu>I$]t!*f!W#CB\LIh\Va+N
	$hC6Z7I5\%G"^A;/#?i[B:d()J>)j*;JmXs6DjmY&CVVg#VnBIs^>P"o-8J')qBUqZ+;#cM,!Orqf2
	8=-7g]@_]AF1Kpom>e"`]Y0M/NpUP;(YGg(=gpll/^\m3[L4rM4,m#K8$V=_#Z'DZMb_\YR]rk0k30
	&VG#EpV]6W-G>4@u4!Zm84<RL2pQ;Qgo(N6d9)1tgU:7G@//j8A`E1RA8Y\*nD:3n3I)Y$\rc&NMuq
	om;e-bZ_`j1eLn=+O=IMJGqgm3X3Mg)],Uc=plNt?X)X*ed>n$8m/5Q$jN/,:Z6RfS^nQ7=j^PQ[[[
	nU@>q1W^M"V;+.eD?cT_5b,Y>!kj1j[Z&B?N/^]+'cOZHh3\9/,cOn-qnFB,N#@')Sr]l@+T.1I,qe
	`=9\=t%%[aYq3:nLXJ-dn`PuA1BTNH5MCLn\uMh[rWhf],,D!p"#)H9Ff(DIU+hH)O)8*;(u8*X\b:
	@A$)l_lTl0p1m<Mk:56H8mO.[1,]\&c4C;o</kNa)q_l,oDlSJVpMh0op?^I\m_r&X.XY#teYi/KV3
	iOp0T>n8@kCN?\=ZI60If([DQguR1R&BdSND&Zom6M`pI]>:?K^\.Zg8Fj(TMrAI.PT-k8i,;q/=P9
	'oDuYoj@*cH_62P/"uhfi3]qMolRR<>&0a5;U:mq2g<qX0UMVfBA><M/cXGgF^BLheU7NZ93WY$7Up
	;/:A#Z"72QXu.G;TD_g9>ge&0:9pK>!PTis\\20H*u:6W=g;Wl#(OuhofRofRVbs,B1LP1\-C+UPS@
	2^O'V%GF+j)#Tc(P^L/CGW)lV=f0L4KG9l;"Zn#SqltHTE217-*\R<bCm^IOsCgh1H8-Dftt7pMF^a
	Eh\VHo1)L;780m:"(eO$arG&G^1sAN!if@S%ofG7t;nlnX8N4S^[&V&/cQPk2]=TM#0VI<i+fXfW42
	1"^rP#,]9PLmiqL)Q[>8g%f\MRK*niT,@@UHA4)hoN&0E7G0UUDiUe*c\]r;kfZ+.24>R&ErM7Z7CG
	j>ljZQH,%bdX`U";I!9t"SP>8;l[W%^RrtmBC+u\F?2EcQVf$9LY0^4m-VD>k0Wi/!f+scs31%SI;5
	@e.+R[T4-']cG[L;GbEk.`)sE9@%DMWB^A6p4I2g':Si?L)YPM6+ZlN)[8n2p2Y:$,uN,D,k!eoed*
	2b&_`i`e<*=OLTDGI*ne*C0->R3]OC[3sb\sa?))G1qfBA)'nc-;Z7!b_5;2+PJflIXaCQ'Gg<0uf;
	d)+g!Q,RJ1RiVO[fkOIIr6Dmau="Q!Ui=23G7r9]W6"IiD;&%ER/C!E,_P!MV(f1f[$lC4&D:,p;-I
	3DuP1+4Wf\Odh$j=U*\?;e>WKB9K?i<0XmEoEh)8O46+\gF#gp/IU#Qc!d@;@Beh(H_7baE#^-_.0$
	\I>>%mEbUQDuOo(/^(`4>.Z1/`,W`[du4.h=8uPhO5m_oFWO*p"UJr+d\,+:nl<Z#W8ne*EL\em.l`
	P#"G6)X>h:,n'mVE6S'p]t)%==T?1NB=%JEu?LCrg_mZHC>C[2JFrq5/BUnqdpG.Rog`,V>;A!%9dr
	6rToea$Yoekn-1op]NMhu<<f@2SS']+M6[p&h^+NV2se<`^*jf[uQpG+LQkX29Qfj$l`4Zr!o*GVCM
	g8l[BdSe)$Z`C5P1EZ#9BBdsrEjEVp%`or88;n8bHUjbtES>PO=I/0^3;u#2;reDU%\TB:Y[F]R-_i
	WF!<S#_2+1J"SDX5no>f&0?MB]XfDFNdcqYF,P2N.7?5JWs#@Z*H>7Zh/p<UB4.6=J+Sl;SFOL4of'
	H9ENF'[:6Mj+)q+#4_)J0m1!uqQ+XALEHj/Jq-lPW<h>4fQ]$%_D=aJGMdg='Lhf2cf1\>qKb7iBXc
	`e>'Bm!+i?j4XsA/6@2Y5mA$*^i1p1U"!fo=$3-D5jXl]^$a6AQe6q0XFfiB8m#@"ri@>pUG+L\_m-
	O9d*+J@,$=C$n6#]7'*;p2oEi'Xi]KCA]Tc3:;QT:$hoH'(-"p/;;UN569E(f4+86ZTQB\?6BtMM24
	p7O)AN+X2pK>h;:-6'YNNddS_596lL%+,\/a'>$KTJp)L=,'A/kA7K+pK$],Orj-'Op&!d1&%QrT%#
	m`lKa[ba9p0%%fm1'`(p7Mk-lf"%?#UgB0nk0h[)MWi3*%R](f$^s=Bb9pf6poU!.qTr+rSB+Y,h&?
	n2s_#jf79:QB>c-&&7f7I<rRJH(O%UcJ7Kh>*$1<EKTAEOIuMo5G<jlgJMr98_r+sR'j>o9r'?oqj]
	km+"3gSP0i[lVq*f-oiqDFV%=*=@gE0(>VjmqR^B78GR@phE=@h]@VNr^dI:7?E:A8Z=Rs+$c`RXql
	u;g.8&#R#i3KlV&BBMZ1rI4H7@Z;2;qgUs4$,NG]3UXN[M6oPY;'X(r-)rB->RB5-dinBLNDY<"!t<
	rDl2L(WS`4(kBHb!CL?]+f,n(.^(4'Td;1^T2OiL9IJ^gclg(>CG5EoZG&d$<0N+YoihMPuRcm[aEM
	=-O_=J*.f]C=t8=j#EcnH<t30#;BKZ-C)_Lb:kmT1b<LMe`YDcg6Qf[paM[=C(;f/5*1\?)ifS3#cj
	H%f4qC!,l3/rq&:io.e1,Y:>)<b#]PKmLF=*FCr8s7/sPpq&ch#P[]EXO;\#Jfq#3mFJCD#F6C>QeL
	j([+k>#kFM(>[KT+<YP_Ye8F#s7eUKQc_J/f@cY(EC"MqgZ8P<#+7cAV&ost[.L]W@W8kPCPN"J0J&
	OhR*1;i%aWA_Aj'o_[%*1o3ndHfoT3>eLs!f'^P>S\(V>Z[,#"ehETS(tW"O3PpDXL5.4XqHI4@9E;
	bH(Ifse14TaS8A7\pr+mub?st>`/-MKlhg[q1o6Fq?WDX6;5d=M3Y$/rOf!<3f*]NVf#Am60cn\s0G
	J7ZU!Y:*K8&'tn0#3[EAZ7DZs#%&4F"XX?[T]WgUCa@HWt6mRKPo!rI/+T[A('p+:.%>T(tMt+E8Sr
	ou7*O@h=cN(Jp6lFg:o\c_rDJnga3*OpZg(NDA`)FHkJL4JgNkZG!rrQX>3`kF[7L]mKM;F?6M)XH7
	#HK\F\VCo0AGkD<mbVIS>+RuYB%Iqu/s=XM-*&J5oLA%=NZ3o=/=_^oh,7#[#Hg70E[f\LDM[C*u_h
	gWc/a,fJ,3Wb-+qr52iGb-;%8IYLE`pL!liCfHcVp4]^bL/4Aqb!jEcsNn5<i;M+SJS-gdd-SiF?9&
	?4fl%Z;8gW"MKk^?CMR_!V5:#;X]A)S0ii'A`g@uTV9]NFHZ'-Y+8Hs:8h5*(PYt<J4F`TmW@4Tp,"
	]JP:Dp:BUhP]aMC=@,jcrp:I@i>j`Uc2p>$=PLf@RJNNQ)XXo9)m5q'=kNfob=o5jE'23@F2Eno5LV
	^)F#^e&E5kTnL-.;FL^u$t0unk0+]jh^:u.XL7&5T%%M_FCAV:8\JI1C2Ih4KOHhO&NM:IP*brnGq\
	fRB,%.1z8OZBBY!QNJ
	ASCII85End
End
